/*
 * ToGrammar.cpp
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#include "ToGrammar.h"
#include "ToGrammarRightRG.h"
#include <exception/AlibException.h>

namespace automaton {

namespace convert {

grammar::Grammar ToGrammar::convert(const automaton::Automaton& automaton) {
	grammar::Grammar* out = NULL;
	automaton.getData().Accept((void*) &out, ToGrammar::TO_GRAMMAR);
	grammar::Grammar res = std::move(*out);
	delete out;
	return res;
}

void ToGrammar::Visit(void*, const automaton::EpsilonNFA& ) const {
	throw exception::AlibException("Unsupported automaton type EpsilonNFA");
}

void ToGrammar::Visit(void*, const automaton::MultiInitialStateNFA& ) const {
	throw exception::AlibException("Unsupported automaton type MultiInitialStateNFA");
}

void ToGrammar::Visit(void* data, const automaton::NFA& automaton) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(ToGrammarRightRG::convert(automaton));
}

void ToGrammar::Visit(void* data, const automaton::DFA& automaton) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(ToGrammarRightRG::convert(automaton));
}

void ToGrammar::Visit(void*, const automaton::ExtendedNFA& ) const {
	throw exception::AlibException("Unsupported automaton type ExtendedNFA");
}

void ToGrammar::Visit(void*, const automaton::CompactNFA& ) const {
	throw exception::AlibException("Unsupported automaton type CompactNFA");
}

void ToGrammar::Visit(void*, const DPDA&) const {
	throw exception::AlibException("Unsupported automaton type DPDA");
}

void ToGrammar::Visit(void*, const SinglePopDPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
}

void ToGrammar::Visit(void*, const InputDrivenDPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenDPDA");
}

void ToGrammar::Visit(void*, const InputDrivenNPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
}

void ToGrammar::Visit(void*, const VisiblyPushdownDPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
}

void ToGrammar::Visit(void*, const VisiblyPushdownNPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
}

void ToGrammar::Visit(void*, const RealTimeHeightDeterministicDPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
}

void ToGrammar::Visit(void*, const RealTimeHeightDeterministicNPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
}

void ToGrammar::Visit(void*, const NPDA&) const {
	throw exception::AlibException("Unsupported automaton type NPDA");
}

void ToGrammar::Visit(void*, const SinglePopNPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void ToGrammar::Visit(void*, const OneTapeDTM&) const {
	throw exception::AlibException("Unsupported automaton type OneTapeDTM");
}

const ToGrammar ToGrammar::TO_GRAMMAR;

} /* namespace convert */

} /* namespace automaton */
