/*
 * ToGrammarLeftRG.cpp
 *
 *  Created on: 9. 2. 2014
 *      Author: Tomas Pecka
 */

#include "ToGrammarLeftRG.h"
#include <map>

#include <alphabet/LabeledSymbol.h>

#include <exception/AlibException.h>

namespace automaton {

namespace convert {

grammar::LeftRG ToGrammarLeftRG::convert(const automaton::NFA& automaton)
{
	std::map<automaton::State, alphabet::Symbol> nonterminalMap;
	// step 2
	grammar::LeftRG grammar(alphabet::symbolFrom("S"));

	// step 1
	grammar.setTerminalAlphabet(automaton.getInputAlphabet());

	for(const auto& state : automaton.getStates())
	{
		alphabet::Symbol nt = alphabet::createUniqueSymbol(alphabet::Symbol(alphabet::LabeledSymbol(state.getName())), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
		grammar.addNonterminalSymbol(nt);
		nonterminalMap.insert(std::pair<automaton::State, alphabet::Symbol>(state, nt));
	}


	// step 3 - create set of P in G
	for(const auto& transition : automaton.getTransitions())
	{
		const automaton::State& from = transition.first.first;
		const alphabet::Symbol& input = transition.first.second;

		for(const auto& to : transition.second)
		{
			// 3a
			grammar.addRule(nonterminalMap.find(to)->second, std::make_pair(nonterminalMap.find(from)->second, input));

			if(automaton.getFinalStates().count(to) > 0)
				grammar.addRule(grammar.getInitialSymbol(), std::make_pair(nonterminalMap.find(from)->second, input));


			if(automaton.getInitialState() == from)
			{
				grammar.addRule(nonterminalMap.find(to)->second, input);

				if(automaton.getFinalStates().count(to) > 0)
					grammar.addRule(grammar.getInitialSymbol(), input);
			}
		}
	}

	if(automaton.getFinalStates().count(automaton.getInitialState()) > 0)
		grammar.setGeneratesEpsilon(true);

	return grammar;
}

grammar::LeftRG ToGrammarLeftRG::convert(const automaton::DFA& automaton)
{
	std::map<automaton::State, alphabet::Symbol> nonterminalMap;
	// step 2
	grammar::LeftRG grammar(alphabet::symbolFrom("S"));

	// step 1
	grammar.setTerminalAlphabet(automaton.getInputAlphabet());

	for(const auto& state : automaton.getStates())
	{
		alphabet::Symbol nt = alphabet::createUniqueSymbol(alphabet::Symbol(alphabet::LabeledSymbol(state.getName())), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
		grammar.addNonterminalSymbol(nt);
		nonterminalMap.insert(std::pair<automaton::State, alphabet::Symbol>(state, nt));
	}


	// step 3 - create set of P in G
	for(const auto& transition : automaton.getTransitions())
	{
		const automaton::State& from = transition.first.first;
		const alphabet::Symbol& input = transition.first.second;
		const automaton::State& to = transition.second;

		// 3a
		grammar.addRule(nonterminalMap.find(to)->second, std::make_pair(nonterminalMap.find(from)->second, input));

		if(automaton.getFinalStates().count(to) > 0)
			grammar.addRule(grammar.getInitialSymbol(), std::make_pair(nonterminalMap.find(from)->second, input));


		if(automaton.getInitialState() == from)
		{
			grammar.addRule(nonterminalMap.find(to)->second, input);

			if(automaton.getFinalStates().count(to) > 0)
				grammar.addRule(grammar.getInitialSymbol(), input);
		}
	}

	if(automaton.getFinalStates().count(automaton.getInitialState()) > 0)
		grammar.setGeneratesEpsilon(true);

	return grammar;
}

grammar::Grammar ToGrammarLeftRG::convert(const automaton::Automaton& automaton) {
	grammar::Grammar* out = NULL;
	automaton.getData().Accept((void*) &out, ToGrammarLeftRG::TO_GRAMMAR_LEFT_RG);
	grammar::Grammar res = std::move(*out);
	delete out;
	return res;
}

void ToGrammarLeftRG::Visit(void*, const automaton::EpsilonNFA& ) const {
	throw exception::AlibException("Unsupported automaton type EpsilonNFA");
}

void ToGrammarLeftRG::Visit(void*, const automaton::MultiInitialStateNFA& ) const {
	throw exception::AlibException("Unsupported automaton type MultiInitialStateNFA");
}

void ToGrammarLeftRG::Visit(void* data, const automaton::NFA& automaton) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->convert(automaton));
}

void ToGrammarLeftRG::Visit(void* data, const automaton::DFA& automaton) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->convert(automaton));
}

void ToGrammarLeftRG::Visit(void*, const automaton::ExtendedNFA& ) const {
	throw exception::AlibException("Unsupported automaton type ExtendedNFA");
}

void ToGrammarLeftRG::Visit(void*, const automaton::CompactNFA& ) const {
	throw exception::AlibException("Unsupported automaton type CompactNFA");
}

const ToGrammarLeftRG ToGrammarLeftRG::TO_GRAMMAR_LEFT_RG;

} /* namespace convert */

} /* namespace automaton */
