/*
 * Result.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _AUTOMATON_RESULT_H__
#define _AUTOMATON_RESULT_H__

#include <automaton/Automaton.h>
#include <string/LinearString.h>
#include <label/Label.h>

namespace automaton {

namespace run {

class Result : public automaton::VisitableAutomatonBase::const_visitor_type {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static label::Label result(const automaton::Automaton& automaton, const string::LinearString& string);

	static label::Label result(const automaton::DFA& automaton, const string::LinearString& string);
	static label::Label result(const automaton::NFA& automaton, const string::LinearString& string);
	static label::Label result(const automaton::DPDA& automaton, const string::LinearString& string);
private:
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const MultiInitialStateNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;

	static const Result RESULT;
};

} /* namespace run */

} /* namespace automaton */

#endif /* _AUTOMATON_RESULT_H__ */
